@@ -538,7 +538,7 @@ int catalog_open(IsoStream *stream)
     data = stream->data;
     
     if (data->offset != -1) {
-        return ISO_FILE_ALREADY_OPENNED;
+        return ISO_FILE_ALREADY_OPENED;
     }
     
     memset(data->buffer, 0, BLOCK_SIZE);
@@ -563,7 +563,7 @@ int catalog_close(IsoStream *stream)
     data = stream->data;
     
     if (data->offset == -1) {
-        return ISO_FILE_NOT_OPENNED;
+        return ISO_FILE_NOT_OPENED;
     }
     data->offset = -1;
     return ISO_SUCCESS;
@@ -589,7 +589,7 @@ int catalog_read(IsoStream *stream, void *buf, size_t count)
     data = stream->data;
     
     if (data->offset == -1) {
-        return ISO_FILE_NOT_OPENNED;
+        return ISO_FILE_NOT_OPENED;
     }
     
     len = MIN(count, BLOCK_SIZE - data->offset);
@@ -617,12 +617,6 @@ void catalog_get_id(IsoStream *stream, unsigned int *fs_id, dev_t *dev_id,
     *ino_id = 0;
 }
 
-static
-char *catalog_get_name(IsoStream *stream)
-{
-    return strdup("El-Torito Boot Catalog");
-}
-
 static
 void catalog_free(IsoStream *stream)
 {
@@ -630,13 +624,14 @@ void catalog_free(IsoStream *stream)
 }
 
 IsoStreamIface catalog_stream_class = {
+    0,
+    "boot",
     catalog_open,
     catalog_close,
     catalog_get_size,
     catalog_read,
     catalog_is_repeatable,
     catalog_get_id,
-    catalog_get_name,
     catalog_free
 };
 
