@@ -13,9 +13,11 @@
 
 #include <stdlib.h>
 #include <string.h>
+#include <limits.h>
 
 ino_t serial_id = (ino_t)1;
 ino_t mem_serial_id = (ino_t)1;
+ino_t cut_out_serial_id = (ino_t)1;
 
 typedef struct
 {
@@ -123,14 +125,6 @@ void fsrc_get_id(IsoStream *stream, unsigned int *fs_id, dev_t *dev_id,
     *ino_id = data->ino_id;
 }
 
-static
-char *fsrc_get_name(IsoStream *stream)
-{
-    FSrcStreamData *data;
-    data = (FSrcStreamData*)stream->data;
-    return iso_file_source_get_path(data->src);
-}
-
 static
 void fsrc_free(IsoStream *stream)
 {
@@ -141,13 +135,14 @@ void fsrc_free(IsoStream *stream)
 }
 
 IsoStreamIface fsrc_stream_class = {
+    0,
+    "fsrc",
     fsrc_open,
     fsrc_close,
     fsrc_get_size,
     fsrc_read,
     fsrc_is_repeatable,
     fsrc_get_id,
-    fsrc_get_name,
     fsrc_free
 };
 
@@ -181,7 +176,7 @@ int iso_file_source_stream_new(IsoFileSource *src, IsoStream **stream)
         return ISO_OUT_OF_MEM;
     }
     data = malloc(sizeof(FSrcStreamData));
-    if (str == NULL) {
+    if (data == NULL) {
         free(str);
         return ISO_OUT_OF_MEM;
     }
@@ -218,6 +213,193 @@ int iso_file_source_stream_new(IsoFileSource *src, IsoStream **stream)
     return ISO_SUCCESS;
 }
 
+struct cut_out_stream
+{
+    IsoFileSource *src;
+
+    /* key for file identification inside filesystem */
+    dev_t dev_id;
+    ino_t ino_id;
+    off_t offset; /**< offset where read begins */
+    off_t size; /**< size of this file */
+    off_t pos; /* position on the file for read */
+};
+
+static
+int cut_out_open(IsoStream *stream)
+{
+    int ret;
+    struct stat info;
+    IsoFileSource *src;
+    struct cut_out_stream *data;
+    
+    if (stream == NULL) {
+        return ISO_NULL_POINTER;
+    }
+    
+    data = stream->data;
+    src = data->src;
+    ret = iso_file_source_stat(data->src, &info);
+    if (ret < 0) {
+        return ret;
+    }
+    ret = iso_file_source_open(src);
+    if (ret < 0) {
+        return ret;
+    }
+    
+    {
+        off_t ret;
+        if (data->offset > info.st_size) {
+            /* file is smaller than expected */
+            ret = iso_file_source_lseek(src, info.st_size, 0);
+        } else {
+            ret = iso_file_source_lseek(src, data->offset, 0);
+        }
+        if (ret < 0) {
+            return (int) ret;
+        }
+    }
+    data->pos = 0;
+    if (data->offset + data->size > info.st_size) {
+        return 3; /* file smaller than expected */
+    } else {
+        return ISO_SUCCESS;
+    }
+}
+
+static
+int cut_out_close(IsoStream *stream)
+{
+    IsoFileSource *src;
+    if (stream == NULL) {
+        return ISO_NULL_POINTER;
+    }
+    src = ((struct cut_out_stream*)stream->data)->src;
+    return iso_file_source_close(src);
+}
+
+static
+off_t cut_out_get_size(IsoStream *stream)
+{
+    struct cut_out_stream *data = stream->data;
+    return data->size;
+}
+
+static
+int cut_out_read(IsoStream *stream, void *buf, size_t count)
+{
+    struct cut_out_stream *data = stream->data;
+    count = (size_t)MIN(data->size - data->pos, count);
+    if (count == 0) {
+        return 0;
+    }
+    return iso_file_source_read(data->src, buf, count);
+}
+
+static
+int cut_out_is_repeatable(IsoStream *stream)
+{
+    /* reg files are always repeatable */
+    return 1;
+}
+
+static
+void cut_out_get_id(IsoStream *stream, unsigned int *fs_id, dev_t *dev_id,
+                ino_t *ino_id)
+{
+    FSrcStreamData *data;
+    IsoFilesystem *fs;
+    
+    data = (FSrcStreamData*)stream->data;
+    fs = iso_file_source_get_filesystem(data->src);
+
+    *fs_id = fs->get_id(fs);
+    *dev_id = data->dev_id;
+    *ino_id = data->ino_id;
+}
+
+static
+void cut_out_free(IsoStream *stream)
+{
+    struct cut_out_stream *data = stream->data;
+    iso_file_source_unref(data->src);
+    free(data);
+}
+
+IsoStreamIface cut_out_stream_class = {
+    0,
+    "cout",
+    cut_out_open,
+    cut_out_close,
+    cut_out_get_size,
+    cut_out_read,
+    cut_out_is_repeatable,
+    cut_out_get_id,
+    cut_out_free
+};
+
+int iso_cut_out_stream_new(IsoFileSource *src, off_t offset, off_t size, 
+                           IsoStream **stream)
+{
+    int r;
+    struct stat info;
+    IsoStream *str;
+    struct cut_out_stream *data;
+
+    if (src == NULL || stream == NULL) {
+        return ISO_NULL_POINTER;
+    }
+    if (size == 0) {
+        return ISO_WRONG_ARG_VALUE;
+    }
+
+    r = iso_file_source_stat(src, &info);
+    if (r < 0) {
+        return r;
+    }
+    if (!S_ISREG(info.st_mode)) {
+        return ISO_WRONG_ARG_VALUE;
+    }
+    if (offset > info.st_size) {
+        return ISO_FILE_OFFSET_TOO_BIG;
+    }
+    
+    /* check for read access to contents */
+    r = iso_file_source_access(src);
+    if (r < 0) {
+        return r;
+    }
+
+    str = malloc(sizeof(IsoStream));
+    if (str == NULL) {
+        return ISO_OUT_OF_MEM;
+    }
+    data = malloc(sizeof(struct cut_out_stream));
+    if (data == NULL) {
+        free(str);
+        return ISO_OUT_OF_MEM;
+    }
+
+    /* take a new ref to IsoFileSource */
+    data->src = src;
+    iso_file_source_ref(src);
+    
+    data->offset = offset;
+    data->size = MIN(info.st_size - offset, size);
+    
+    /* get the id numbers */
+    data->dev_id = (dev_t) 0;
+    data->ino_id = cut_out_serial_id++;
+
+    str->refcount = 1;
+    str->data = data;
+    str->class = &cut_out_stream_class;
+
+    *stream = str;
+    return ISO_SUCCESS;
+}
+
 
 
 typedef struct
@@ -237,7 +419,7 @@ int mem_open(IsoStream *stream)
     }
     data = (MemStreamData*)stream->data;
     if (data->offset != -1) {
-        return ISO_FILE_ALREADY_OPENNED;
+        return ISO_FILE_ALREADY_OPENED;
     }
     data->offset = 0;
     return ISO_SUCCESS;
@@ -252,7 +434,7 @@ int mem_close(IsoStream *stream)
     }
     data = (MemStreamData*)stream->data;
     if (data->offset == -1) {
-        return ISO_FILE_NOT_OPENNED;
+        return ISO_FILE_NOT_OPENED;
     }
     data->offset = -1;
     return ISO_SUCCESS;
@@ -281,7 +463,7 @@ int mem_read(IsoStream *stream, void *buf, size_t count)
     data = stream->data;
     
     if (data->offset == -1) {
-        return ISO_FILE_NOT_OPENNED;
+        return ISO_FILE_NOT_OPENED;
     }
     
     if (data->offset >= data->size) {
@@ -311,12 +493,6 @@ void mem_get_id(IsoStream *stream, unsigned int *fs_id, dev_t *dev_id,
     *ino_id = data->ino_id;
 }
 
-static
-char *mem_get_name(IsoStream *stream)
-{
-    return strdup("[MEMORY SOURCE]");
-}
-
 static
 void mem_free(IsoStream *stream)
 {
@@ -327,13 +503,14 @@ void mem_free(IsoStream *stream)
 }
 
 IsoStreamIface mem_stream_class = {
+    0,
+    "mem ",
     mem_open,
     mem_close,
     mem_get_size,
     mem_read,
     mem_is_repeatable,
     mem_get_id,
-    mem_get_name,
     mem_free
 };
 
@@ -411,7 +588,12 @@ off_t iso_stream_get_size(IsoStream *stream)
 inline
 int iso_stream_read(IsoStream *stream, void *buf, size_t count)
 {
-    return stream->class->read(stream, buf, count);
+    int ret;
+
+    ret= stream->class->read(stream, buf, count);
+    if(ret<0)
+        return ISO_FILE_READ_ERROR;
+    return ret;
 }
 
 inline
@@ -427,8 +609,19 @@ void iso_stream_get_id(IsoStream *stream, unsigned int *fs_id, dev_t *dev_id,
     stream->class->get_id(stream, fs_id, dev_id, ino_id);
 }
 
-inline
-char *iso_stream_get_name(IsoStream *stream)
+void iso_stream_get_file_name(IsoStream *stream, char *name)
 {
-    return stream->class->get_name(stream);
+    char *type = stream->class->type;
+    
+    if (!strncmp(type, "fsrc", 4)) {
+        FSrcStreamData *data = stream->data;
+        char *path = iso_file_source_get_path(data->src);
+        strncpy(name, path, PATH_MAX);
+    } else if (!strncmp(type, "boot", 4)) {
+        strcpy(name, "BOOT CATALOG");
+    } else if (!strncmp(type, "mem ", 4)) {
+        strcpy(name, "MEM SOURCE");
+    } else {
+        strcpy(name, "UNKNOWN SOURCE");
+    }
 }
